package
{
import flash.display.Sprite;
import flash.events.Event;
import flash.events.TimerEvent;
import flash.geom.Matrix;
import flash.system.System;
import flash.text.TextField;
import flash.utils.Timer;
import flash.utils.getTimer;

/**
 * 
 * @author renaun
 */
public class OptimizeIt extends Sprite
{
	/**
	 * 
	 */
	public function OptimizeIt(isCached:Boolean = false)
	{	
		var shape:Sprite = new Sprite();
		shape.graphics.beginFill(0xECECEC, 1);
		shape.graphics.drawRect(0, 0, 480, 320);
		shape.graphics.endFill();
		shape.cacheAsBitmap = true;
		if (isCached)
			shape.cacheAsBitmapMatrix = new Matrix();
		stage.addChild(shape);
	}
	
	/**
	 * 	Common
	 */
	public var pointerX:Number = 0;
	public var pointerY:Number = 0;
	
	
	private var fps:TextField = new TextField();
	protected var fpsLogicCount:int = 0;
	protected var fpsRenderCount:int = 0;
	private var fpsStartTime:Number = 0;
	private var fpsStartMem:uint = 0;
	
	/**
	 * 
	 */
	private function initFPS():void
	{
		fpsStartTime = (new Date()).getTime();
		fpsLogicCount = 0;
		fpsRenderCount = 0;
		fps.width = 240;
		fps.y = 300;
	}
	/**
	 * 
	 */
	public function initMemory():void
	{
		fpsStartMem = System.totalMemory;
	}
	
	/**
	 * 
	 */
	protected function showFPS():void
	{
		if (!stage.contains(fps))
		{
			var logicCount:int = fpsLogicCount - 1;
			var memory:int = System.totalMemory-fpsStartMem>>10;
			var now:Number = (new Date()).getTime();
			var dif:Number = ((now - fpsStartTime) / 1000);
			fps.text = "FPS: " + (fpsRenderCount / dif).toFixed(1) + "/" + (logicCount / dif).toFixed(1) + " C: " + fpsRenderCount+"/"+logicCount + " MEM: " + memory;
			stage.addChild(fps);
		}
		else
		{
			initFPS();
			initMemory();
			stage.removeChild(fps);
		}
	
	}

	public static var FRAME_RATE:int = 20;
	
	private var timer:Timer;
	
	private const time:Number = 1000 / FRAME_RATE;
	private var beforeTime:int = 0;
	private var afterTime:int = 0;
	private var sleepTime:int = 0;
	private var overSleepTime:int = 0;
	private var excess:int = 0;
	
	protected const testDuration:int = 501;
	
	
	public function start(enterFrameMode:Boolean = false):void
	{
		if (enterFrameMode)
		{
			initFPS();
			stage.addEventListener(Event.ENTER_FRAME, enterFrameHandler);
			return;
		}
		if (!timer)
		{
			timer = new Timer(time, 1);
			timer.addEventListener(TimerEvent.TIMER, timerHandler);
		}
		beforeTime = 0;
		afterTime = getTimer()+time; // Set the first after time to now + time, so first frame doesn't skip
		sleepTime = 0;
		overSleepTime = 0;
		excess = 0;		
		timer.delay = time;
		timer.reset();
		initFPS();
		timer.start();
	}
	private function enterFrameHandler(event:Event):void
	{
		event.stopImmediatePropagation();
		logic();
		render();
	}
	
	public function pause():void
	{
		if (timer)
		{
			timer.removeEventListener(TimerEvent.TIMER, timerHandler);
			timer.stop();
		}
		if (stage.hasEventListener(Event.ENTER_FRAME))
			stage.removeEventListener(Event.ENTER_FRAME, enterFrameHandler);
	}
	
	private function timerHandler(event:TimerEvent):void
	{
		event.stopImmediatePropagation();
		beforeTime = getTimer();
		overSleepTime = (beforeTime - afterTime) - sleepTime;
		// Game Specific Calls
		logic();
		render();
		afterTime = getTimer();
		sleepTime = time - afterTime + beforeTime - overSleepTime;
		// If it slept longer then our frame time add to excess
		if (sleepTime <= 0) {
			excess -= sleepTime
			sleepTime = 2;
		}        
		// Setup new timer delay
		timer.reset();
		timer.delay = sleepTime;
		timer.start();
		// Handle logic calls for each frame time we are lagging
		while (excess > time) {
			logic();
			excess -= time;
		}
		// Force Flash Player to start a new Frame
		event.updateAfterEvent();
	}
	
	public function logic():void
	{
		if (fpsLogicCount > testDuration)
		{
			pause();
			return;
		}
		fpsLogicCount++;
	}
	
	public function render():void
	{
		fpsRenderCount++;
	}
	
}
}