﻿/**
* TextLog by Grant Skinner. Feb 2, 2010
* Visit www.gskinner.com/blog for documentation, updates and more free code.
*
*
* Copyright (c) 2010 Grant Skinner
* 
* Permission is hereby granted, free of charge, to any person
* obtaining a copy of this software and associated documentation
* files (the "Software"), to deal in the Software without
* restriction, including without limitation the rights to use,
* copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the
* Software is furnished to do so, subject to the following
* conditions:
* 
* The above copyright notice and this permission notice shall be
* included in all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
* OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
* HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
* WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
* OTHER DEALINGS IN THE SOFTWARE.
**/
package com.gskinner.performance {
	
	import flash.events.Event;
	
	/**
	* Simple example showing tracing the output from PerformanceTest as it is generated.
	**/
	public class TextLog {
		
	// public properties:
		/**
		* This function is called each time new text is logged, with a single param containing the new text.
		* Set to use trace by default.
		**/
		public var out:Function = trace;
		
		/**
		* The full generated text from all of the tests run while this log has been active.
		**/
		public var text:String;
		
	// protected properties:
		protected var performanceTest:PerformanceTest;
		protected var currentTestSuite:TestSuite;
		
		
	// constructor:
		/**
		* Creates a new TextLog instance. You can specify the PerformanceTest instance to log, or null to use
		* the global instance.
		**/
		public function TextLog(performanceTest:PerformanceTest=null) {
			this.performanceTest = performanceTest ? performanceTest : PerformanceTest.getInstance();
			this.performanceTest.addEventListener(Event.COMPLETE,handleComplete);
			text = "";
		}
		
	// protected methods:
		/** @private **/
		protected function log(str:String):void {
			if (text.length > 0) { text += "\n"; }
			text += str;
			out(str);
		}
	
	// event handlers:
		/** @private **/
		protected function handleComplete(evt:Event):void {
			var pf:PerformanceTest = performanceTest;
			if (pf.currentTestSuite != currentTestSuite) {
				currentTestSuite = pf.currentTestSuite;
				if (currentTestSuite) {
					log(currentTestSuite.toString().split("\n")[0]);
				}
			}
			log( (currentTestSuite ? "\t" : "")+pf.currentTest.toString() );
		}
		
		

	}
	
}
